package recipe

import (
	"context"
	"fmt"

	"coopcloud.tech/abra/cli/internal"
	"coopcloud.tech/abra/pkg/autocomplete"
	"coopcloud.tech/abra/pkg/formatter"
	"coopcloud.tech/abra/pkg/lint"
	"coopcloud.tech/abra/pkg/log"
	"github.com/urfave/cli/v3"
)

var recipeLintCommand = cli.Command{
	Name:            "lint",
	Usage:           "Lint a recipe",
	Aliases:         []string{"l"},
	UsageText:       "abra recipe lint [options] <recipe>",
	HideHelpCommand: true,
	Flags: []cli.Flag{
		internal.OnlyErrorFlag,
		internal.ChaosFlag,
	},
	Before:                internal.SubCommandBefore,
	EnableShellCompletion: true,
	ShellComplete:         autocomplete.RecipeNameComplete,
	Action: func(ctx context.Context, cmd *cli.Command) error {
		recipe := internal.ValidateRecipe(cmd)

		if err := recipe.Ensure(internal.Chaos, internal.Offline); err != nil {
			log.Fatal(err)
		}

		tableCol := []string{"ref", "rule", "severity", "satisfied", "skipped", "resolve"}
		table := formatter.CreateTable(tableCol)

		hasError := false
		bar := formatter.CreateProgressbar(-1, "running recipe lint rules...")
		for level := range lint.LintRules {
			for _, rule := range lint.LintRules[level] {
				if internal.OnlyErrors && rule.Level != "error" {
					log.Debugf("skipping %s, does not have level \"error\"", rule.Ref)
					continue
				}

				skipped := false
				if rule.Skip(recipe) {
					skipped = true
				}

				skippedOutput := "-"
				if skipped {
					skippedOutput = "✅"
				}

				satisfied := false
				if !skipped {
					ok, err := rule.Function(recipe)
					if err != nil {
						log.Warn(err)
					}

					if !ok && rule.Level == "error" {
						hasError = true
					}

					if ok {
						satisfied = true
					}
				}

				satisfiedOutput := "✅"
				if !satisfied {
					satisfiedOutput = "❌"
					if skipped {
						satisfiedOutput = "-"
					}
				}

				table.Append([]string{
					rule.Ref,
					rule.Description,
					rule.Level,
					satisfiedOutput,
					skippedOutput,
					rule.HowToResolve,
				})

				bar.Add(1)
			}
		}

		if table.NumLines() > 0 {
			fmt.Println()
			table.Render()
		}

		if hasError {
			log.Warn("watch out, some critical errors are present in your recipe config")
		}

		return nil
	},
}
